//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  thermodl.c - This utility uses TMEX to download the results of the
//               current mission of a DS1921 Thermochron iButton.
//
//  Compiler:  Microsoft Visual C 
//  Version:   1.02
//
//  History:  1.00 -> 1.02  Reduced size of malloc memory.
//            1.02 -> 1.03  Y2K update, display all histogram bins, debug
//                          dump

#define TMEXUTIL

#include "ibtmex.h"   
#include "thermo.h"

// external function prototypes 
extern int  RunThermoScript(ThermoState script[],FILE *fp);
extern void MissionStatusToString(MissionStatus *, int, char *);
extern void InterpretStatus(MissionStatus *);
extern void InterpretHistogram(Histogram *);
extern void HistogramToString(Histogram *, int, char *);
extern void InterpretAlarms(TempAlarmEvents *, MissionStatus *);
extern void AlarmsToString(TempAlarmEvents *, char *);
extern void InterpretLog(Log *, MissionStatus *);
extern void LogToString(Log *, int, char *);
extern void LogToFile(Log *, int, FILE *);  // (1.01)
extern void DebugToString(MissionStatus *, TempAlarmEvents *, Histogram *, Log *, char *); // (1.03)
// local function prototypes
void PrintHeader(char *);
void PrintResults(FILE *,int,int);


//----------------------------------------------------------------------
//  This is the Main routine for thermodl.
//
void main(short argc, char **argv)
{
   int Fahrenheit=FALSE,filenum,dodebug,i;
   FILE *fp;

   // check arguments to see if request instruction with '?' or too many
   if ((argc > 4) || ((argc > 1) && (argv[1][0] == '?' || argv[1][1] == '?')))
   {
       printf("\nusage: thermodl <output_filename> </Fahrenheit> </Debug>\n"
              "  - Thermochron download on the default\n"
              "    1-Wire port (use SetPort32.exe to set)\n"
              "  - <output_filename> optional output filename\n"
              "  - </Fahrenheit> optional Fahrenheit mode (default Celsius)\n"
              "  - </Debug> enable optional debug data dump at end of download\n"
              "  - press any key to stop download\n"
              "  - version 1.03\n");
       exit(1);
   }

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum,&PortType))
   {
       printf("ERROR, Could not read the default PortNum and PortType!\n");
       exit(1);
   }

   // print the header info 
   PrintHeader("thermodl Version 1.03");

   // check arguments (1.03)
   Fahrenheit = FALSE;
   dodebug = 0;
   filenum = 0;
   for (i = 1; i < argc; i++)
   {
      if (argv[i][0] != '/')
         filenum = i;
      else if ((argv[i][1] == 'D') || (argv[i][1] == 'd'))
         dodebug = 1;
      else if ((argv[i][1] == 'F') || (argv[i][1] == 'f'))
         Fahrenheit = TRUE;
   }

   // open the output file  
   fp = NULL;
   if (filenum > 0)
   {
      fp = fopen(argv[filenum],"w+");
      if(fp == NULL)
      {    
         printf("ERROR, Could not open output file!\n");
         exit(1);
      }
      else
         printf("File '%s' opened to write mission results.\n",
                 argv[filenum]);
   }

   // run the script to find and download thermochron
   if (!RunThermoScript(DownloadThermo,stdout))
   { 
      printf("Thermochon not downloaded, end program\n");
      exit(1);
   }
   
   // print the output
   PrintResults(fp,Fahrenheit,dodebug);

   // close opened file
   if (fp != NULL)
   {
      printf("File '%s' closed.\n",
              argv[filenum]);
      fclose(fp);
   }

   printf("End program normally\n");
   exit(0);
}


//--------------------------------------------------------------------------
//  Prints a message, and the current driver versions.
//
void PrintHeader(char *msg)
{
   char tstr[80];       

   // print a header with driver version info   
   printf("\n%s\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-2000\n\n",msg);
   printf("Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("Main driver version: %s\n",tstr);
   printf("Sub-driver version, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
   {
      printf("No Hardware Driver for this type found!");
      exit(1);
   }
   printf(" %s\n\n\n",tstr);
}


//--------------------------------------------------------------------------
//  Prints the mission data optionaly to a file or standard out
//
void PrintResults(FILE *fp, int ConvertToF, int dodebug)
{
   char *str;  

   // check if need to use standard out
   if (fp == NULL)
      fp = stdout;

   // get big block to use as a buffer
   str = malloc((unsigned int)57000);  // (1.01) 
   if (str == NULL)
   {
      printf("Insufficient memory available to print!\n"); 
      return;
   }

   // mission status 
   InterpretStatus(&MissStat);
   MissionStatusToString(&MissStat, ConvertToF, &str[0]);
   fprintf(fp,"\n%s\n",str);

   // alarm events
   InterpretAlarms(&AlarmData, &MissStat);
   AlarmsToString(&AlarmData, &str[0]);
   fprintf(fp,"%s\n",str);

   // histogram
   InterpretHistogram(&HistData);
   HistogramToString(&HistData, ConvertToF, &str[0]);
   fprintf(fp,"%s\n",str);

   // log data
   InterpretLog(&LogData, &MissStat);
   LogToString(&LogData, ConvertToF, &str[0]); 
   fprintf(fp,"%s\n",str);

   // debug data (1.03)
   if (dodebug)
   {
      DebugToString(&MissStat, &AlarmData, &HistData, &LogData, &str[0]); 
      fprintf(fp,"%s\n",str);
   }

   // free the memory block used
   free(str);
}
